@@ -15,7 +15,6 @@
 from oslo_log import log as logging
 
 from os_brick import initiator
-
 from os_brick.initiator.connectors import fibre_channel
 from os_brick.initiator import linuxfc
 
@@ -87,12 +86,17 @@ class FibreChannelConnectorS390X(fibre_channel.FibreChannelConnector):
         ]
         return host_device
 
-    def _remove_devices(self, connection_properties, devices, device_info):
+    def _remove_devices(self, connection_properties, devices, device_info,
+                        force, exc):
         hbas = self._linuxfc.get_fc_hbas_info()
         targets = connection_properties['targets']
-        possible_devs = self._get_possible_devices(hbas, targets)
+        addressing_mode = connection_properties.get('addressing_mode')
+        possible_devs = self._get_possible_devices(hbas, targets,
+                                                   addressing_mode)
         for platform, pci_num, target_wwn, lun in possible_devs:
             target_lun = self._get_lun_string(lun)
-            self._linuxfc.deconfigure_scsi_device(pci_num,
-                                                  target_wwn,
-                                                  target_lun)
+            with exc.context(force, 'Removing device %s:%s:%s failed',
+                             pci_num, target_wwn, target_lun):
+                self._linuxfc.deconfigure_scsi_device(pci_num,
+                                                      target_wwn,
+                                                      target_lun)
