@@ -20,7 +20,7 @@ import glob
 import os
 import re
 import time
-from typing import Any, Iterable, Optional, Union
+from typing import Any, Iterable, Optional, TypedDict, Union
 
 from oslo_concurrency import processutils as putils
 from oslo_log import log as logging
@@ -39,6 +39,15 @@ from os_brick import utils
 LOG = logging.getLogger(__name__)
 
 
+class ConnData(TypedDict):
+    stop_connecting: bool
+    num_logins: int
+    failed_logins: int
+    stopped_threads: int
+    found_devices: list[str]
+    just_added_devices: list[str]
+
+
 class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
     """Connector class to attach/detach iSCSI volumes."""
 
@@ -170,7 +179,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
 
     def _get_all_targets(
             self,
-            connection_properties: dict) -> list[tuple[str, str, list]]:
+            connection_properties: dict) -> \
+            list[tuple[str, str, list | int | str]]:
         addressing_mode = connection_properties.get('addressing_mode')
         res = super()._get_all_targets(connection_properties)
 
@@ -510,6 +520,10 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
                             connection_properties)})
             raise exception.VolumePathsNotFound()
 
+    def supports_multipath(self):
+        return self._linuxscsi.is_multipath_running(
+            root_helper=self._root_helper)
+
     @utils.trace
     @utils.connect_volume_prepare_result
     @base.synchronized('connect_volume', external=True)
@@ -529,6 +543,7 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         target_lun(s) - LUN id of the volume
         Note that plural keys may be used when use_multipath=True
         """
+        self.check_multipath(connection_properties)
         try:
             if self.use_multipath:
                 return self._connect_multipath_volume(connection_properties)
@@ -709,10 +724,15 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
         last_try_on = 0.0
         found: list = []
         just_added_devices: list = []
+
         # Dict used to communicate with threads as detailed in _connect_vol
-        data = {'stop_connecting': False, 'num_logins': 0, 'failed_logins': 0,
-                'stopped_threads': 0, 'found_devices': found,
-                'just_added_devices': just_added_devices}
+
+        data: ConnData = {'stop_connecting': False,
+                          'num_logins': 0,
+                          'failed_logins': 0,
+                          'stopped_threads': 0,
+                          'found_devices': found,
+                          'just_added_devices': just_added_devices}
 
         ips_iqns_luns = self._get_ips_iqns_luns(connection_properties)
         # Launch individual threads for each session with the own properties
@@ -792,6 +812,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
             self._linuxscsi.wait_for_mpath_device(mpath)
             if not wwn:
                 wwn = self._linuxscsi.get_sysfs_wwn(found, mpath)
+            LOG.debug('Found WWN %(wwn)s for multipath device %(mpath)s',
+                      {'wwn': wwn, 'mpath': mpath})
 
         assert wwn is not None
         return self._get_connect_result(connection_properties, wwn, found,
@@ -977,7 +999,8 @@ class ISCSIConnector(base.BaseLinuxConnector, base_iscsi.BaseISCSIConnector):
 
     def _munge_portal(
             self,
-            target: tuple[str, str, Union[list, str]]) -> tuple[str, str, str]:
+            target: tuple[str, str, Union[list, str, int]]) -> \
+            tuple[str, str, list | str | int]:
         """Remove brackets from portal.
 
         In case IPv6 address was used the udev path should not contain any
