@@ -18,14 +18,12 @@ import binascii
 import copy
 from unittest import mock
 
-import six
-
 from castellan.common.objects import symmetric_key as key
 from castellan.tests.unit.key_manager import fake
+
 from os_brick.encryptors import cryptsetup
 from os_brick import exception
 from os_brick.tests.encryptors import test_base
-from oslo_concurrency import processutils as putils
 
 
 def fake__get_key(context, passphrase):
@@ -113,7 +111,7 @@ class CryptsetupEncryptorTestCase(test_base.VolumeEncryptorTestCase):
                                 root_helper=self.root_helper,
                                 connection_info=connection_info,
                                 keymgr=fake.fake_api())
-        self.assertIn(type, six.text_type(exc))
+        self.assertIn(type, str(exc))
 
     @mock.patch('os_brick.executor.Executor._execute')
     @mock.patch('os.path.exists', return_value=True)
@@ -156,33 +154,6 @@ class CryptsetupEncryptorTestCase(test_base.VolumeEncryptorTestCase):
         mock_execute.assert_called_once_with(
             'cryptsetup', 'status', wwn, run_as_root=True)
 
-    @mock.patch('os_brick.executor.Executor._execute')
-    def test_attach_volume_unmangle_passphrase(self, mock_execute):
-        fake_key = '0725230b'
-        fake_key_mangled = '72523b'
-        self.encryptor._get_key = mock.MagicMock()
-        self.encryptor._get_key.return_value = fake__get_key(None, fake_key)
-
-        mock_execute.side_effect = [
-            putils.ProcessExecutionError(exit_code=2),  # luksOpen
-            mock.DEFAULT,
-            mock.DEFAULT,
-        ]
-
-        self.encryptor.attach_volume(None)
-
-        mock_execute.assert_has_calls([
-            mock.call('cryptsetup', 'create', '--key-file=-', self.dev_name,
-                      self.dev_path, process_input=fake_key,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-            mock.call('cryptsetup', 'create', '--key-file=-', self.dev_name,
-                      self.dev_path, process_input=fake_key_mangled,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-            mock.call('ln', '--symbolic', '--force',
-                      '/dev/mapper/%s' % self.dev_name, self.symlink_path,
-                      root_helper=self.root_helper, run_as_root=True,
-                      check_exit_code=True),
-        ])
-        self.assertEqual(3, mock_execute.call_count)
+    def test_extend_volume(self):
+        self.assertRaises(NotImplementedError,
+                          self.encryptor.extend_volume, mock.sentinel.context)
